<?php
/**
 * Copyright © 2015 FiloBlu . All rights reserved.
 */

namespace FiloBlu\Flow\Helper;

class Attribute extends \Magento\Framework\App\Helper\AbstractHelper
{
    /**
     * @var
     */
    protected $_connection;
    /**
     * @var
     */
    protected $_setup;
    /**
     * @var \Monolog\Logger
     */
    protected $_logger;

    /**
     * Attribute constructor.
     * @param \Magento\Framework\App\Helper\Context $contenxt
     * @param LoggerProvider $loggerProvider
     */
    public function __construct(
        \Magento\Framework\App\Helper\Context $contenxt,
        \FiloBlu\Flow\Helper\LoggerProvider $loggerProvider
    ) {
        $this->_logger = $loggerProvider->getLogger();
        parent::__construct($contenxt);
    }

    /**
     * @param $option
     * @return bool
     * @throws \Exception
     */
    public function addAttributeOption($option)
    {

        $optionTable = 'eav_attribute_option';
        $attributeId = $option['attribute_id'];
        $optionValue = $option['value'];
        $storeId = $option['store_id'];

        if (empty($optionValue)) {
            throw new \Exception("Option value cannot be empty");
        }

        if ($storeId != 0) {
            throw new \Exception("I can only add option for admin store view");
        }

        if (empty($attributeId)) {
            throw new \Exception("Attribute id has to be defined when adding an option");
        }
        // add option
        $data = array(
            'attribute_id' => $option['attribute_id']
        );
        $this->getConnection()->insert($optionTable, $data);
        $optionId = $this->getConnection()->lastInsertId($optionTable);
        if (empty($optionId)) {
            return false;
        }
        $optionValueTable = 'eav_attribute_option_value';
        $option['option_id'] = $optionId;
        unset($option['attribute_id']);
        $this->getConnection()->insert($optionValueTable, $option);
        return $optionId;
    }

    /**
     * @return mixed
     */
    protected function getConnection()
    {
        if (empty($this->_connection)) {
            $this->_connection = $this->getSetup()->getConnection(\Magento\Framework\App\ResourceConnection::DEFAULT_CONNECTION);
        }
        return $this->_connection;
    }

    /**
     * @return mixed
     */
    protected function getSetup()
    {
        if (empty($this->_setup)) {
            $this->_setup = \Magento\Framework\App\ObjectManager::getInstance()->get('\Magento\Framework\App\ResourceConnection');
        }
        return $this->_setup;
    }

    /**
     * @param $option
     * @return bool
     * @throws \Exception
     */
    public function addAttributeTranslation($option)
    {
        $optionId = $option['option_id'];
        $optionValue = $option['value'];
        $storeId = $option['store_id'];

        if (empty($optionValue)) {
            throw new \Exception("Option value cannot be empty");
        }

        if (empty($storeId)) {
            throw new \Exception("I can't add translation for default store view");
        }

        if (empty($optionId)) {
            throw new \Exception("I can't add translation for unknown option");
        }

        $optionValueTable = 'eav_attribute_option_value';
        //we'd might update
        $select = $this->getConnection()->select()
            ->from($optionValueTable)
            ->where('option_id = ' . $optionId)
            ->where('store_id = ' . $storeId);

        $result = $this->getConnection()->fetchRow($select);
        if ($result) {
            $where = array(
                'option_id =?' => $option['option_id'],
                'store_id =?' => $option['store_id']
            );
            $this->getConnection()->update($optionValueTable, $option, $where);
            return true;
        }
        $this->getConnection()->insert($optionValueTable, $option);
        return true;
    }

}
