<?php

namespace FiloBlu\Flow\Model\From;

/**
 * Class Eavs
 * @package FiloBlu\Flow\Model\From
 */
class Eavs extends AbstractFrom
{
    /**
     * @var integer
     */
    const PRODUCT_NOT_FOUND_ERR_CODE = 1;

    /**
     * @var integer
     */
    const SKU_ATTRIBUTE_ERR_CODE = 2;

    /**
     * @var integer
     */
    const DROPDOWN_STORE_ERR_CODE = 3;

    /**
     * @var string
     */
    const ENTITY_MANAGER_CLASS_PREFIX = 'FiloBlu\Flow\Model\From\Entity\\';

    /**
     * @var string
     */
    const DEFAULT_ENTITY_TYPE_CODE = 'catalog_product';

    /**
     * @var string
     */
    const ATTRIBUTE_ENTITY_TYPE_SUFFIX = 'attribute';

    /**
     * @var string
     */
    const PRODUCT_ATTRIBUTE_ENTITY_TYPE_CODE = 'product_attribute';

    /**
     * @var string
     */
    const EMAIL_FROM = 'Eavs processor <magenteam@.it>';

    /**
     * @var string
     */
    const EMAIL_TO = '';

    /**
     * @var integer
     */
    const PRODUCT_REF_ID = 0;
    /**
     * @var integer
     */
    const ATTRIBUTE_REF_ID = 1;

    /**
     * @var integer
     */
    const SPECIAL_REF_ID = 2;

    /**
     * @var string
     */
    const CONFIG_FLOW_BULKMODE = 'filoblu_flow/flow_process/flow_update_bulkmode';
    /**
     * @var string
     */
    const CONFIG_FLOW_BULKTHRESHOLD = 'filoblu_flow/flow_process/flow_update_bulkmodethreshold';
    /**
     * @var string
     */
    const EXECUTED_ON_ATTRIBUTE = 'attribute';

    /**
     * @var string
     */
    const EXECUTED_ON_PRODUCT = 'product';

    /**
     * @var bool
     */
    protected $_dontRetryErrorCodes = true;

    /**
     * @var int
     */
    protected $_productEntityTypeId = 4;

    /**
     * @var array
     */
    protected $_excludeFromBulk = ['sku'];

    /**
     * @var Entity\Bulk|null
     */
    protected $_bulkModel = null;

    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface|null
     */
    protected $_scopeConfig = null;


    protected $_helperFlow;
    /**
     * @param \Magento\Framework\Model\Context $context
     * @param \Magento\Framework\Registry $registry
     * @param \Magento\Framework\Model\ResourceModel\AbstractResource $resource
     * @param \Magento\Framework\Data\Collection\Db $resourceCollection
     * @param array $data
     */
    public function __construct(
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        \FiloBlu\Flow\Helper\Data $helperFlow,
        \FiloBlu\Flow\Model\From\Entity\Bulk $bulkModel,
        \Magento\Framework\Model\ResourceModel\AbstractResource $resource = null,
        \Magento\Framework\Data\Collection\AbstractDb $resourceCollection = null,
        array $data = []
    ) {
        $this->_helperFlow = $helperFlow;
        $this->_bulkModel = $bulkModel;
        $this->_scopeConfig = $scopeConfig;
        parent::__construct($context, $registry, $resource, $resourceCollection, $data);
    }


    /**
     * @return void
     */
    public function _construct()
    {
        $this->_init('FiloBlu\Flow\Model\ResourceModel\From\Eavs', 'meta_id');
    }

    /**
     * @return $this|bool
     */
    public function process()
    {

        // $this->_logger->info(__METHOD__);

        /* check store view */
        $storeId = $this->getStoreId();
        if ($storeId === false) {
            return $this->error(false, "Store view " . $this->getS() . " is missing");
        }

        $entityTypeCode = $this->getMetaRefId();
        if (empty($entityTypeCode) || $entityTypeCode == self::PRODUCT_REF_ID || $entityTypeCode == self::SPECIAL_REF_ID) { // 0 // Product
            $entityTypeCode = self::DEFAULT_ENTITY_TYPE_CODE;
        } else {
            if ($entityTypeCode == self::ATTRIBUTE_REF_ID) { // 1 // Attribute
                $entityTypeCode = self::PRODUCT_ATTRIBUTE_ENTITY_TYPE_CODE;
            }
        }

        $entityManagerClassName = $this->getEntityManagerClassName($entityTypeCode);
        try {
            $entityManager = \Magento\Framework\App\ObjectManager::getInstance()->create($entityManagerClassName,
                [
                    'data' => [
                        'entityIdentifier' => $this->getE(),
                        'attributeCode' => $this->getA(),
                        'attributeValue' => $this->getV(),
                        'entityTypeCode' => $entityTypeCode,
                        'storeId' => $storeId
                    ]
                ]);
            $entityManager->process();
        } catch (\Exception $e) {
            $this->_logger->info($e->getMessage());
            //if(array_key_exists($e->getCode(),$this->_dontRetryErrorCodes)){
            return $this->error(false, $e->getMessage());
            //}
            return $this->error(true, $e->getMessage());
        }

        return $this;
    }

    /**
     * @return array|bool
     */
    protected function getStoreId()
    {
        $storeView = $this->getS();
        $language = $this->getL();
        $allStore = $this->_helperFlow->getAllStoreIds();
        /* storeview is in s tag with codename */
        if (!empty($storeView)) {
            if (strtolower(trim($storeView)) == 'default') {
                $stores[] = 0;
                return $stores;
            }
            if (isset($allStore[$storeView])) {
                $stores[] = $allStore[$storeView];
                return $stores;
            }
        } elseif (!empty($language)) {
            $languages = $this->_helperFlow->getListLanguageForAllStores();
            if (isset($languages[strtolower($language)])) {
                return $languages[strtolower($language)];
            }
        } else {
            $stores[] = 0;
            return $stores;//Mage_Core_Model_App::ADMIN_STORE_ID; //usually 0
        }

        return false;
    }

    /**
     * @param $entityTypeCode
     * @return string
     */
    protected function getEntityManagerClassName($entityTypeCode)
    {
        if ($entityTypeCode == self::DEFAULT_ENTITY_TYPE_CODE) {
            return self::ENTITY_MANAGER_CLASS_PREFIX . 'Product';
        } elseif (strstr($entityTypeCode, self::ATTRIBUTE_ENTITY_TYPE_SUFFIX) !== false) {
            return self::ENTITY_MANAGER_CLASS_PREFIX . 'Attribute';
        }

        return self::ENTITY_MANAGER_CLASS_PREFIX . 'AbstractEntity';
    }

    /**
     * @param $query
     * @param $file
     * @return mixed
     */
    protected function preprocessFileRows($query, $file)
    {

        $bulk_enabled = $this->_scopeConfig->getValue(self::CONFIG_FLOW_BULKMODE);

        if (!$bulk_enabled) {
            return $query;
        }

        $this->_registry->registry('attribute_configurable_copy');
        $startTime = time();
        try {
            $this->_bulkModel->_insertMissingAttributeOptionValues($file);
            $this->_logger->info("bulk process : missing attribute options inserted in " . (time() - $startTime) . "s");
        } catch (\Exception $e) {
            $this->_logger->info($e->getMessage());
        }


        $startTime = time();
        try {
            $this->_bulkModel->_setMetaStatusToNotFoundProducts($file);
            $this->_logger->info("bulk process : meta status for not found products in " . (time() - $startTime) . "s");
        } catch (\Exception $e) {
            $this->_logger->info($e->getMessage());
        }

        $startTime = time();

        $loop = 0;
        $limit = is_numeric($this->_scopeConfig->getValue(self::CONFIG_FLOW_BULKTHRESHOLD)) ? $this->_scopeConfig->getValue(self::CONFIG_FLOW_BULKTHRESHOLD) : 5000;
        $count = 0;
        $offset = 0;

        while ($rows = $this->_bulkModel->_getMetaAttributesforProducts($file, $limit, $offset)) {

            foreach ($rows as $backend_model => $items) {
                $count = $count + (int)$this->_dumpEAVS($items);
            }

            $loop++;
            $offset = ($limit * $loop) - $count;


        }

        $this->_logger->info("bulk process : values to update got in " . (time() - $startTime) . "s");

        return $query;

    }

    /**
     * @param $items
     * @return bool|int
     */
    protected function _dumpEAVS($items)
    {

        if (empty($items)) {
            return false;
        }

        $targetTable = $items[0]['backend_type'];

        $this->_logger->info("Doing " . sizeof($items) . " of " . $items[0]['backend_type']);

        return $this->_saveAttributes($items, $targetTable);
    }

    /**
     * @param $items
     * @param $targetTable
     * @return bool|int
     */
    protected function _saveAttributes($items, $targetTable)
    {
        if (empty($items)) {
            return false;
        }

        $Connection = $this->getResource()->getConnection();

        $metaIds = array();
        $toUpdate = array();
        for ($i = 0; $i < sizeof($items); $i++) {
            $toUpdate[] = sprintf("(%s, %s, %s, %s)", $items[$i]['attribute_id'], $items[$i]['store_id'],
                $items[$i][$this->getEntityField()], $Connection->quote($items[$i]['v']));
            $metaIds[] = $items[$i]['meta_id'];
        }

        $updateQuery = "INSERT INTO `catalog_product_entity_{$targetTable}`
                            (`attribute_id`, `store_id`, `{$this->getEntityField()}`, `value`)
                        VALUES ";
        $updateQuery .= implode(', ', $toUpdate);
        $updateQuery .= " ON DUPLICATE KEY UPDATE `value` = VALUES(`value`);";

        $metaUpdateQuery = "UPDATE `flow_from_eavs` SET `meta_processed` = 1 WHERE `meta_id` IN (" . implode(', ',
                $metaIds) . ");";

        try {
            $Connection->beginTransaction();
            $Connection->query($updateQuery);
            $Connection->query($metaUpdateQuery);
            $Connection->commit();
            return sizeof($items);
        } catch (Exception $e) {
            Mage::log(__METHOD__ . ':' . __LINE__ . ': Rollback happened.' . $e->getMessage());
            $Connection->rollback();
            return false;
        }
    }

    /**
     * @return string
     */
    protected function getEntityField()
    {

        return 'row_id';
        //$edition = $this->_productMetadata->getEdition();
        return ($edition == 'Enterprise' || $edition == 'B2B') ? 'row_id' : 'entity_id';

    }

}