<?php

namespace FiloBlu\Flow\Model\From\Entity;

/**
 * Class Bulk
 * @package FiloBlu\Flow\Model\From\Entity
 */
class Bulk
{
    /**
     * @var \Magento\Framework\DB\Adapter\AdapterInterface
     */
    protected $_connection;

    /**
     * @var int
     */
    protected $_productEntityTypeId = 4;

    /**
     * @var array
     */
    protected $_excludeFromBulk = ['sku'];

    /**
     * Bulk constructor.
     * @param \Magento\Framework\Model\Context $context
     * @param \Magento\Framework\App\ResourceConnection $connection
     * @param array $data
     */
    public function __construct(
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\App\ResourceConnection $connection,
        array $data = []
    ) {
        $this->_connection = $connection->getConnection();
    }

    /**
     * @param $file
     * @throws \Exception
     */
    public function _insertMissingAttributeOptionValues($file)
    {

        //selects values of dropdown attributes with no option yet
        $query = "SELECT eavalues.value as value, eavalues.attribute_id FROM (SELECT e.v as value, ea.attribute_id
			    	FROM `flow_from_eavs` e
			    	JOIN `eav_attribute` ea
				    	ON e.`a` = ea.`attribute_code`
				    	AND ea.`entity_type_id` = ?
				    	AND ea.`frontend_input` = 'select'
				    	AND (e.s=0 OR e.s is null)
				    	AND e.a NOT REGEXP '^_'
				    	AND e.`meta_processed` = 0
				    	AND e.meta_file = ?
				    	GROUP BY attribute_id,value) AS eavalues
			    	LEFT JOIN eav_attribute_option_value option_value
			    		ON option_value.value = eavalues.value
			    	LEFT JOIN eav_attribute_option eaoption
			    		ON eaoption.attribute_id = eavalues.attribute_id
			    		AND option_value.option_id = eaoption.option_id
			    	WHERE option_value.value_id IS null
			    	GROUP BY eavalues.attribute_id,eavalues.value;";

        $rows = $this->_connection->fetchAll($query, [$this->_productEntityTypeId, $file->getId()]);

        foreach ($rows as $row) {
            $insertOptionQuery = "INSERT INTO eav_attribute_option(attribute_id) VALUES(" . $row['attribute_id'] . ");
		INSERT INTO eav_attribute_option_value(option_id,store_id,value) VALUES(LAST_INSERT_ID(),0," . $this->_connection->quote($row['value']) . ");";
            try {

                $this->_connection->query($insertOptionQuery);

            } catch (Exception $e) {
                throw new \Exception(__METHOD__ . ':' . __LINE__ . ': Rollback happened.' . $e->getMessage());
            }
        }
    }

    /**
     * @param $file
     * @throws \Exception
     */
    public function _setMetaStatusToNotFoundProducts($file)
    {
        $query = "	UPDATE `flow_from_eavs` e
					LEFT JOIN catalog_product_entity as products on products.sku=e.e
					SET e.`meta_processed` = 3
					WHERE e.`meta_processed` = 0
					AND e.a NOT REGEXP '^_'
					AND (e.`meta_ref_id` != 2)
					AND products.sku is null
					AND e.meta_file = {$file->getId()};";
        try {
            $this->_connection->query($query);
        } catch (Exception $e) {
            throw new \Exception(__METHOD__ . ':' . __LINE__ . ': Rollback happened.' . $e->getMessage());
        }
    }

    /**
     * @param $file
     * @param int $limit
     * @param int $offset
     * @return array|bool
     */
    public function _getMetaAttributesforProducts($file, $limit = 0, $offset = 0)
    {

        if ($limit > 0) {
            $limit_query = " LIMIT $limit OFFSET $offset";
        }

        $query = "SELECT cpe.`{$this->getEntityField()}`, e.`v`, IF(cs.`store_id` IS NULL, 0, cs.`store_id`) store_id, ea.`attribute_id`, ea.`backend_type`, ea.`frontend_input`, e.`meta_id`, e.`meta_file` FROM `flow_from_eavs` e
                                JOIN `catalog_product_entity` cpe
                                        ON cpe.`sku` = e.`e`
								        AND e.a NOT REGEXP '^_'
		                                AND e.`meta_processed` = 0
		                                AND meta_file = {$file->getId()}
                                JOIN `eav_attribute` ea
                                        ON e.`a` = ea.`attribute_code`
                                        AND ea.`entity_type_id` = {$this->_productEntityTypeId}
                                        AND
                                            ( (ea.`frontend_input` != 'select' AND ea.`frontend_input` != 'multiselect')
                                                OR
                                               e.a='visibility'
                                            )
                                        AND ea.attribute_code not in ('" . implode("','", $this->_excludeFromBulk) . "')
                                LEFT JOIN `store` cs ON cs.`code` = e.`s`";

        $dropdownQuery = "SELECT cpe.`{$this->getEntityField()}`, option_value.option_id as v, 0 store_id, ea.`attribute_id`, ea.`backend_type`, ea.`frontend_input`, e.`meta_id`, e.`meta_file`
        						FROM `flow_from_eavs` e
                                JOIN `catalog_product_entity` cpe
                                        ON cpe.`sku` = e.`e`
        								AND e.a NOT REGEXP '^_'
        								AND e.a!='visibility'
        								AND (e.s=0 OR e.s IS NULL)
                                		AND e.`meta_processed` = 0
		                                AND meta_file = {$file->getId()}
                                JOIN `eav_attribute` ea
                                        ON e.`a` = ea.`attribute_code`
                                        AND ea.`entity_type_id` = {$this->_productEntityTypeId}
                                        AND ea.`frontend_input` = 'select'
                                        AND ea.attribute_code not in ('" . implode("','", $this->_excludeFromBulk) . "')
                                JOIN eav_attribute_option eaoption
					    			ON eaoption.attribute_id=ea.attribute_id
					    		JOIN eav_attribute_option_value option_value
					    			ON option_value.value=e.v
					    			AND option_value.option_id=eaoption.option_id
					    			AND option_value.store_id=0 $limit_query;";

        $query .= " UNION " . $dropdownQuery;

        $rows = $this->_connection->fetchAll($query);

        $toProcess = array();

        foreach ($rows as $row) {
            if (!array_key_exists($row['backend_type'], $toProcess) || !is_array($toProcess[$row['backend_type']])) {
                $toProcess[$row['backend_type']] = array();
            }
            $toProcess[$row['backend_type']][] = $row;
        }

        return empty($toProcess) ? false : $toProcess;
    }

    /**
     * @return string
     */
    protected function getEntityField()
    {
        return 'row_id';
        //$edition = $this->_productMetadata->getEdition();
        return ($edition == 'Enterprise' || $edition == 'B2B') ? 'row_id' : 'entity_id';
    }
}
