<?php

namespace FiloBlu\Flow\Model\From;

class Stock extends AbstractFrom {

    /**
     * @var Monolog\Logger
     */
    protected $_logger;
    protected $_helper_flow;
    protected $_product_factory;
    protected $_stock_registry;
    protected $_stock_qty;

    /**
     * @param \Magento\Framework\Model\Context $context
     * @param \Magento\Framework\Registry $registry
     * @param \Magento\Catalog\Model\ProductFactory $productFactory
     * @param \FiloBlu\Flow\Helper\LoggerProvider $loggerProvider
     * @param array $data
     */
    public function __construct(
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        \FiloBlu\Flow\Helper\Data $helper_flow,
        \FiloBlu\Flow\Helper\LoggerProvider $loggerProvider,
        \Magento\Catalog\Model\ProductFactory $productFactory,
        \Magento\CatalogInventory\Api\StockRegistryInterface $stockRegistry,
        \FiloBlu\Flow\Helper\Stock $stock_qty,
        \Magento\Framework\App\ProductMetadataInterface $_productMetadata,
        array $data = []
    ) {
        $this->_helper_flow = $helper_flow;
        $this->_logger = $loggerProvider->getLogger();
        $this->_product_factory = $productFactory;
        $this->_stock_registry = $stockRegistry;
        $this->_stock_qty= $stock_qty;
        $this->_productMetadata = $_productMetadata;
        parent::__construct($context, $registry, null, null, $data);
    }

    /**
     * @return void
     */
    public function _construct()
    {
        $this->_init('FiloBlu\Flow\Model\ResourceModel\From\Stock');
    }

    public function process(){

        //  $this->_logger->info(__METHOD__);

        $product_id = $this->_product_factory->create()->getIdBySku($this->getSku());

        //$product = $this->_product_factory->create()->setStoreId(0);

        if(!$product_id){
            $message = sprintf("Product not found! product_code: '%s'", $this->getSku());
            return $this->error(false, $message);
        }

        if($product_id){

            // salvo il prezzo sul prodotto semplice
            //$product->load($product_id);

            $stock = $this->_stock_registry->getStockItem($product_id);

            $_in_stock = false;

            //$this->_logger->info('Process STOCK ---> ' . $product_id);

            try {

                $raw_qty = intval($this->getData('fisico_disponibile'));

                $qty = $this->_stock_qty->getRealQty($product_id,$raw_qty);

                $stock->setQty($qty);

                if ($qty > $stock->getMinQty()){
                    $_in_stock = true;
                    $stock->setIsInStock($_in_stock);
                }
                elseif($stock->getBackorders() != \Magento\CatalogInventory\Model\Stock::BACKORDERS_NO) {
                    $_in_stock = true;
                    $stock->setIsInStock($_in_stock);
                }
                else {
                    $stock->setIsInStock($_in_stock);
                }


                $stock_data = $stock->getData();
                unset($stock_data["type_id"]);
                $this->getResource()->getConnection()->insertOnDuplicate("cataloginventory_stock_item", $stock_data);

                if ($raw_qty != $qty){
                    $preserved_qty = $raw_qty - $qty;
                    $message = sprintf("Preserved Quantity '%s' For Product '%s'", $preserved_qty ,$product_id);
                    $this->error(false, $message);

                }

                $this->_logger->info('Process STOCK - qty ' . $qty . ' ID: '.$product_id);


            } catch(\Exception $e){
                exit($e->getMessage());
                $this->_logger->info('Errore nel salvataggio dello Stock. ' . $e->getMessage() );
            }

        }

        return $this;
    }

    protected function postprocessFileRows($query) {

        try {

            $query_update_child = "
                    UPDATE cataloginventory_stock_item AS cpsi
                            INNER JOIN
                        catalog_product_entity AS cpe ON cpe.entity_id = cpsi.product_id 
                    SET 
                        is_in_stock = IF (qty > min_qty OR backorders != 0 ,1 ,0)
                    WHERE
                        cpe.type_id = 'simple';
                ";

            $this->getResource()->getConnection()->query($query_update_child);

            $query_update_parent = "
                    UPDATE cataloginventory_stock_item AS cpsi
                            INNER JOIN
                        (SELECT 
                            cpe.entity_id AS parent_id,
                                IF(SUM(qty) <= 0, 0, 1) AS in_stock,
                                SUM(qty) AS qty
                        FROM
                            catalog_product_super_link AS cpsl
                        INNER JOIN catalog_product_entity AS cpe ON cpe.{$this->getEntityField()} = cpsl.parent_id
                        INNER JOIN cataloginventory_stock_item AS csi ON csi.product_id = cpsl.product_id
                        GROUP BY cpsl.parent_id) AS table_sum ON table_sum.parent_id = cpsi.product_id 
                    SET 
                        cpsi.is_in_stock = table_sum.in_stock;
                ";

            $this->getResource()->getConnection()->query($query_update_parent);

        } catch(\Exception $e){
            exit($e->getMessage());
            $this->_logger->info('Errore nel salvataggio del Post Stock Item Fix Tuning. ' . $e->getMessage());
            return false;
        }

        return true;

    }

    protected function getEntityField() {

        $edition = $this->_productMetadata->getEdition();
        return ($edition == 'Enterprise') ? 'row_id' : 'entity_id';

    }


}